@@ -111,7 +111,7 @@ char *fake = N_("CrossProduct of two vectors in R^3");
 char *fake = N_("Get the determinantal divisors of an integer matrix (not its characteristic)");
 char *fake = N_("Direct sum of matrices");
 char *fake = N_("Direct sum of a vector of matrices");
-char *fake = N_("Get the eigenvalues of a matrix (Currently only for 2x2 and triangular matrices)");
+char *fake = N_("Get the eigenvalues of a matrix (Currently only for up to 4x4 or triangular matrices)");
 char *fake = N_("Apply the Gram-Schmidt process (to the columns) with respect to inner product given by B (if not given use hermitian product)");
 char *fake = N_("Hankel matrix");
 char *fake = N_("Hilbert matrix of order n");
@@ -124,6 +124,8 @@ char *fake = N_("Test if a vector is in a subspace");
 char *fake = N_("Is a matrix (or number) invertible (Integer matrix is invertible iff it's invertible over the integers)");
 char *fake = N_("Is a matrix (or number) invertible over a field");
 char *fake = N_("Is a matrix normal");
+char *fake = N_("Is a matrix positive definite");
+char *fake = N_("Is a matrix positive semidefinite");
 char *fake = N_("Is a matrix skew-hermitian");
 char *fake = N_("Is a matrix unitary");
 char *fake = N_("Get the jordan block corresponding to lambda and n");
@@ -138,8 +140,8 @@ char *fake = N_("Get the orthogonal complement of the columnspace");
 char *fake = N_("Return pivot columns of a matrix, that is columns which have a leading 1 in rref form, also returns the row where they occur");
 char *fake = N_("Projection of vector v onto subspace W given a sesquilinear form B (if not given use hermitian product)");
 char *fake = N_("Get the QR decomposition of A, returns R and Q can be a reference");
-char *fake = N_("Return the Raleigh Quotient of a matrix and a vector");
 char *fake = N_("Get the rank of a matrix");
+char *fake = N_("Return the Rayleigh quotient of a matrix and a vector");
 char *fake = N_("Rosser matrix, a classic symmetric eigenvalue test problem");
 char *fake = N_("Rotation around origin in R^2");
 char *fake = N_("Rotation around origin in R^3 about the x-axis");
@@ -223,6 +225,7 @@ char *fake = N_("Find root of a function using the Muller's method");
 char *fake = N_("Find root of a function using the secant method");
 char *fake = N_("Find roots of a polynomial (given as vector of coefficients)");
 char *fake = N_("Find roots of a quadratic polynomial (given as vector of coefficients)");
+char *fake = N_("Find roots of a quartic polynomial (given as vector of coefficients)");
 char *fake = N_("Use classical non-adaptive Runge-Kutta of fourth order method to numerically solve y'=f(x,y) for initial x0,y0 going to x1 with n increments, returns y at x1");
 char *fake = N_("Calculate average of an entire matrix");
 char *fake = N_("Integral of the GaussFunction from 0 to x (area under the normal curve)");
