@@ -1,5 +1,5 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2002 George Lebl
+ * Copyright (C) 1997-2007 George Lebl
  *
  * Author: George Lebl
  *
@@ -68,7 +68,7 @@ internal_matrix_free (GelMatrix *m)
 #ifdef MATRIX_DEBUG
 	/*debug*/printf ("%s\n", G_GNUC_PRETTY_FUNCTION);
 #endif
-	
+
 	if (m->use == 1) {
 #ifdef MATRIX_DEBUG
 		/*debug*/printf ("ACTUALLY FREE\n");
@@ -133,6 +133,8 @@ gel_matrixw_new(void)
 	m->cached_value_only_real = 0;
 	m->cached_value_only_rational = 0;
 	m->cached_value_only_integer = 0;
+	m->cached_value_or_bool_only = 0;
+	m->rref = 0;
 	
 	m->tr = 0;
 	m->regx = NULL;
@@ -162,6 +164,8 @@ gel_matrixw_new_with_matrix(GelMatrix *mat)
 	m->cached_value_only_real = 0;
 	m->cached_value_only_rational = 0;
 	m->cached_value_only_integer = 0;
+	m->cached_value_or_bool_only = 0;
+	m->rref = 0;
 	
 	m->tr = 0;
 	m->regx = NULL;
@@ -381,6 +385,10 @@ gel_matrixw_set_size (GelMatrixW *m, int nwidth, int nheight)
 		height = nheight;
 	}
 
+	/* we're changing things so make sure we don't consider it rref
+	 * anymore */
+	m->rref = 0;
+
 	if (m->regw >= width &&
 	    m->regh >= height) {
 		/*if we're the sole owner, we'll have to zero out some things*/
@@ -546,6 +554,10 @@ gel_matrixw_get_region (GelMatrixW *m, int *regx, int *regy, int w, int h)
 	
 	new = gel_matrixw_copy (m);
 
+	/* we're changing things so make sure we don't consider it rref
+	 * anymore */
+	new->rref = 0;
+
 	if (getmax (regx, w) >= new->regw ||
 	    getmax (regy, h) >= new->regh) {
 		/* FIXME: maybe we can whack a bit of region stuff here,
@@ -848,14 +860,14 @@ gel_matrixw_diagonalof (GelMatrixW *source)
 	len = MIN(width,height);
 
 	mm = gel_matrix_new ();
-	gel_matrix_set_size (mm, len, 1, FALSE /* padding */);
+	gel_matrix_set_size (mm, 1, len, FALSE /* padding */);
 
 	for (i = 0; i < len; i++) {
 		GelETree *n = gel_matrixw_set_index (source, i, i);
 		if (n != NULL)
 			n = copynode (n);
 
-		gel_matrix_index (mm, i, 0) = n;
+		gel_matrix_index (mm, 0, i) = n;
 	}
 		
 	return gel_matrixw_new_with_matrix (mm);
@@ -873,6 +885,11 @@ gel_matrixw_transpose(GelMatrixW *m)
 #endif
 	
 	new = gel_matrixw_copy(m);
+
+	/* we're changing things so make sure we don't consider it rref
+	 * anymore */
+	new->rref = 0;
+
 	new->tr = !new->tr;
 	return new;
 }
@@ -890,6 +907,8 @@ internal_make_private (GelMatrixW *m, int w, int h)
 	m->cached_value_only_real = 0;
 	m->cached_value_only_rational = 0;
 	m->cached_value_only_integer = 0;
+	m->cached_value_or_bool_only = 0;
+	m->rref = 0;
 
 #ifdef MATRIX_DEBUG
 	printf ("Make private %p %d %d\n", m, w, h);
@@ -943,8 +962,14 @@ gel_matrixw_free(GelMatrixW *m)
 	
 	mf = (GelMatrixWFreeList *)m;
 	
+#ifdef MEM_DEBUG_FRIENDLY
+
+	memset (mf, 0xaa, sizeof (GelMatrixW));
+	g_free (mf);
+#else
 	mf->next = free_matrices;
 	free_matrices = mf;
+#endif
 }
 
 void
